# Program to generate a regression model based on scikit-learn methods
# The input file is a CSV file where the first columns are for explanatory variables and
# the last column is for the response variable for activity information.
#
# Prof. Walter F. de Azevedo Jr.
# azevedolab. net
#

def get_number_of_cols(csv_file_in):
    """Function to get the number of columns in a CSV file"""
    
    # Import libraries
    import csv
    import sys
    
    # Try to open CSV file
    try:
        fo1 = open(csv_file_in,"r")
        csv1 = csv.reader(fo1)
    except IOError:
        sys.exit("IOError! I can't find "+csv_file_in+" file!")
    
    # Get the number of columns
    for line in csv1:
        n_cols = len(line)
        break
    
    return n_cols

def calc_RMSE(y_obs,y_pred):
    """Function to calculate root-mean squared error(RMSE)"""
    # Import library
    import numpy as np

    # Get the length of the array    
    n = len(y_obs)
    
    # Make sure we have arrays
    yo = np.array(y_obs)
    yp = np.array(y_pred)
    
    # Get the mean of experimental activities
    yo_mean = np.mean(yo)
    
    # Looping through arrays
    rmse0 = 0.0
    for i in range(n):
        rmse0 = (yp[i] - yo_mean)**2

    # Calculate RMSE    
    rmse = np.sqrt(rmse0/(n-1))
    
    return rmse

def calc_chi2(y_obs,y_pred):
    """Function to calculate chi-squared"""
    
    # Import library
    import numpy as np

    # Get the length of the array    
    n = len(y_obs)
    
    # Make sure we have arrays
    yo = np.array(y_obs)
    yp = np.array(y_pred)

    # Looping through arrays
    chi2 = 0.0
    for i in range(n):
        if yp[i] != 0:
            chi2 += np.abs(((yo[i]-yp[i])**2)/yp[i])
        
    return chi2
        
def cross_validated_LOO_r2(n,method_in):
    """Function to call SF for LOO files"""
    import numpy as np
    
    # Set uo empty list
    q = []
    
    # Looping through CSV files
    for i in range(1000):
        # Fix the number of characters
        my_str_3_char = str(i)
        if len(my_str_3_char)< 4:
            my_str_lig = str((3-len(my_str_3_char) )*"0")+my_str_3_char
            my_str = my_str_lig # We must have three columns
        else:
            my_str = my_str_3_char
        
        # Try to open
        try:
            csv_file_in = "loo_file_"+my_str+".csv"
            fo1 = open(csv_file_in,"r")
            fo1.close()
            
            # Call SF() to get q2 values
            q.append(SF_CV(csv_file_in,n,method_in))
            
        except IOError:
            break       # Break once finished
    
    q_array =  np.array(q)
    q2 = np.mean(q_array)        # It returns the mean we could easily change to minimum
    
    return q2
        
def clean_line(line_in):
    """Function to get rid of [ ] ' and spaces"""
    line0 = str(line_in)
    line00 = line0.replace(" ","")
    line000 = line00.replace("[","")
    line0000 = line000.replace("]","")
    line00000 = line0000.replace("'","")
    
    return line00000 

def generate_LOO_files(csv_in,n):
    """Function to generate LOO files for cross-validated r2"""
    import csv
    import sys
    
    my_list = []
    
    # Try to open csv_in
    try:
        fo1 =  open(csv_in,"r")
        csv1 = csv.reader(fo1)
    except IOError:
        sys.exit("IOError!")
    
    # Get first line
    for line in csv1:
        first_line =  line
        break
    
    # Get data
    for line in csv1:
        my_list.append(line)  
    fo1.close()

    for i in range(len(my_list)):
        # Write LOO files
        # Fix the number of characters
        my_str_3_char = str(i)
        if len(my_str_3_char)< 4:
            my_str_lig = str((3-len(my_str_3_char) )*"0")+my_str_3_char
            my_str = my_str_lig # We must have three columns
        else:
            my_str = my_str_3_char
        fo1 = open("loo_file_"+my_str+".csv","w")
        aux0 = clean_line(str(first_line))
        fo1.write(aux0+"\n")
        for j in range(len(my_list)):
            if i != j:
                aux1 = clean_line(str(my_list[j])) 
                fo1.write(aux1+"\n")
        fo1.close()

def SF_CV(my_scoring_function_file,num_ind_var,my_mlr_method_in):
    """ Scoring function (SF) function and returns R2"""
    
    # Import libraries
    import scikit_regression_methods_v001z_works as reg_method001     
    import numpy as np
    
    # Calls function read_csv_4_SF()
    my_first_line, num_var, x,y = read_csv_4_SF(my_scoring_function_file,num_ind_var)
    
    # Calls filter_data_4_SF
    my_indep_variables,z = filter_data_4_SF(x,y,num_var)

    # Creates regression model (return self.b, self.R2, self.R2adj,self.sd,self.F,self.Fpv)
    SF_model = reg_method001.Multvar_reg_modeling(num_var,y,z,'y',my_indep_variables,my_mlr_method_in)     
    n_obs,my_model_coefs,my_R,my_R2,my_R_p_value,my_R2_adj,my_sd,my_rho,my_rho_p,\
    my_f_stat,my_errors,Q_factor,y_pred = SF_model.summary()
    
    # Double my_errors values
    my_d_errors = np.zeros(len(my_errors) ,dtype=float)
    for i in range(len(my_errors)):
        my_d_errors[i] = 2*my_errors[i]    
    
    # Calls function to generate models
    p1,my_model_string0 = gen_model_4_SF(num_var,my_model_coefs,my_first_line,x,y) 
       
    # Return R
    return my_R2
    
def SF(my_scoring_function_file,num_ind_var,my_mlr_method_in):
    """ Scoring function (SF) function"""
    
    # Import libraries
    import scikit_regression_methods_v001z_works as reg_method001      
    import numpy as np
    
    # Append to SF.log file
    SF_log_fo = open("SF.log","w")
    SF_csv_fo = open("SF.csv","w")

    # Calls function read_csv_4_SF()
    my_first_line, num_var, x,y = read_csv_4_SF(my_scoring_function_file,num_ind_var)
    
    # Calls filter_data_4_SF
    my_indep_variables,z = filter_data_4_SF(x,y,num_var)

    # Creates regression model (return self.b, self.R2, self.R2adj,self.sd,self.F,self.Fpv)
    SF_model = reg_method001.Multvar_reg_modeling(num_var,y,z,'y',my_indep_variables,my_mlr_method_in)     
    n_obs,my_model_coefs,my_R,my_R2,my_R_p_value,my_R2_adj,my_sd,my_rho,my_rho_p,\
    my_f_stat,my_errors,Q_factor,y_pred = SF_model.summary()
    
    # Double my_errors values
    my_d_errors = np.zeros(len(my_errors) ,dtype=float)
    for i in range(len(my_errors)):
        my_d_errors[i] = 2*my_errors[i]    
    
    # Calls function to generate models
    p1,my_model_string0 = gen_model_4_SF(num_var,my_model_coefs,my_first_line,x,y) 
    
    # Call generate_LOO_files()
    #generate_LOO_files(z,y,my_mlr_method_in,my_first_line)
    
    # Clean my_model_string0
    my_model_string1 = my_model_string0.replace(" ","")
    my_model_string2 = my_model_string1.replace("++","+")
    my_model_string3 = my_model_string2.replace("+-","-")
    my_model_string4 = my_model_string3.replace("-"," - ")
    my_model_string5 = my_model_string4.replace("+"," + ")
    
    # Call cross_validated_LOO_r2()
    my_q2_loo = cross_validated_LOO_r2(num_ind_var,"LinearRegression")
    
    # Call calc_chi2()
    my_chi2 = calc_chi2(y,y_pred)
    
    # Call calc_RMSE()
    my_RMSE = calc_RMSE(y,y_pred)
    
    # Write results
    SF_log_fo.write("\n\n######################################### MODEL GENERATOR ###############################################\n")
    SF_log_fo.write("\n\nFile: "+my_scoring_function_file)
    SF_log_fo.write("\n\nRegression equation : \nPred. "+str(my_first_line[len(my_first_line)-1])+" = "+my_model_string5)
    for i in range(len(my_d_errors)):
        if i == 0:
            SF_log_fo.write('\nError for c(0)\t:% -5.6f  '%my_d_errors[i] )
        else:
            SF_log_fo.write('\nError for '+my_first_line[i-1]+'\t:% -5.6f  '%my_d_errors[i]  ) 
    SF_log_fo.write("\n\nBest fit equation between observed and predicted information:"+str(p1))
    SF_log_fo.write("\n\nN                    : "+str(n_obs))
    SF_log_fo.write('\nStandard deviation   :% -5.6f' % my_sd)
    SF_log_fo.write('\nR                    :% -5.6f' % my_R )
    SF_log_fo.write('\nR-squared            :% -5.6f' % my_R2 )
    SF_log_fo.write("\np-value              :% -5.6e" % my_R_p_value)
    SF_log_fo.write('\nAdjusted R-square    :% -5.6f' % my_R2_adj )
    SF_log_fo.write('\nSpearman correlation :% -5.6f' % my_rho )
    SF_log_fo.write('\np-value              :% -5.6e' % my_rho_p )
    SF_log_fo.write('\nQuality factor (Q)   :% -5.6f' % Q_factor )
    SF_log_fo.write('\nq-squared for LOO    :% -5.6f' % my_q2_loo )
    SF_log_fo.write('\nF-stat               :% -5.6f' % my_f_stat )
    SF_log_fo.write('\nChi-squared          :% -5.6f' % my_chi2 )
    SF_log_fo.write('\nRMSE                 :% -5.6f' % my_RMSE )
    
    # Set up my_rho_p to 0.000 for  my_rho_p < 0.0001
    if my_rho_p < 0.0001:
        my_rho_p = 0.000
    SF_csv_fo.write(my_scoring_function_file+","+\
                          str(my_R2)+","+str(my_R2_adj)+","+str(my_sd)+","+\
                          str(my_rho)+","+str(my_rho_p)+"\n")
    
    
def read_csv_4_SF(my_scoring_function_file_in,num_ind_var_in):
    """Function to read CSV file"""
    
    # Import libraries
    import csv
    import numpy as np
    
    # Reads input file name and polynomial equation degree
    input_file_name = my_scoring_function_file_in
    var = num_ind_var_in
    
    # Reads first line to get headers
    csv_in = open(input_file_name,"r") 
    get_csv_lines = csv.reader(csv_in)
    
    # Reads CSV file first line only
    for line in get_csv_lines:     
        break
    csv_in.close()
    
    # Reads CSV file and skips first line
    csv = np.genfromtxt (input_file_name, delimiter=",", skip_header = 1) 
    
    # Gets each column from CSV file
    y = csv[:,var] 
    x = csv[:,0:var]
    
    return line,var, x,y
    
def filter_data_4_SF(x,y,var):
    """Function to filter data"""
    import numpy as np
    
    # Defines numbers of rows and columns 
    columns = var
    rows = len(y)

    # Sets up a matrix
    z = np.array([[0]*columns]*rows,float)           # np.array([[0]*column]*row,float)

    # looping through x to get independent variable columns
    for i in range(rows):
        for j in range(columns):
            z[i,j] = x[i,j]
        
    # Looping through to create a list with independent variables
    my_indep_variables = []
    for i in range(columns):
        my_indep_variables.append("x"+str(i+1))
 
    return my_indep_variables,z

def gen_model_4_SF(var,regression_coefs,first_line,x_in,y_in):
    """Function to generate models"""
    import numpy as np
    
    # Looping through to generate model
    p = 0
    total_variables = var + 1
    SF_model_string = ""
    for i in range(total_variables):
        if i == 0:
            p += regression_coefs[i]
            SF_model_string += str("%12.6f"%regression_coefs[i])+"+" 
        else:
            p += regression_coefs[i]*x_in[:,i-1]
            SF_model_string += str("+%12.6f"%regression_coefs[i])+"*("+first_line[i-1]+")"
    
    # Generate best fit line for experimental and computational model (least-squares polynomial fitting) 
    z = np.polyfit(y_in,p, 1)
    p1 = np.poly1d(z)
  
    return p1,SF_model_string   
    
def main():
    
    # Get the CSV file name, we the following file to be used as imput:   
    # Verma_Hansch_EJMC_2010_Table1.csv
    # Verma_Hansch_EJMC_2010_Table2.csv
    # Verma_Hansch_EJMC_2010_Table3.csv    
    # cdk_ki.csv
    # data01_19col.csv
    
    csv_file = "cdk_ki.csv"
    
    # Call get_number_of_cols()
    n_columns = get_number_of_cols(csv_file)
        
    # Call generate_LOO_files()
    generate_LOO_files(csv_file,n_columns)
    
    # Call SF()
    SF(csv_file,n_columns-1,"ElasticNetCV")         
    
main()